@@ -11,10 +11,10 @@ from datetime import datetime, date, timedelta, tzinfo
 import time
 import calendar
 import tempfile
+import warnings
 from webob.util.dictmixin import DictMixin
 from webob.datastruct import EnvironHeaders
 from webob.multidict import MultiDict, UnicodeMultiDict, NestedMultiDict, NoVars
-from webob.useragent import UserAgent, parse_search_query
 from webob.etag import AnyETag, NoETag, ETagMatcher, IfRange, NoIfRange
 from webob.headerdict import HeaderDict
 from webob.statusreasons import status_reasons
@@ -229,6 +229,45 @@ def _rfc_reference(header, section):
     return "  For more information on %s see `section %s <%s>`_." % (
         header, section, link)
 
+class deprecated_property(object):
+    """
+    Wraps a decorator, with a deprecation warning or error
+    """
+    def __init__(self, decorator, attr, message, warning=True):
+        self.decorator = decorator
+        self.attr = attr
+        self.message = message
+        self.warning = warning
+
+    def __get__(self, obj, type=None):
+        if obj is None:
+            return self
+        self.warn()
+        return self.decorator.__get__(obj, type)
+
+    def __set__(self, obj, value):
+        self.warn()
+        self.decorator.__set__(obj, value)
+
+    def __delete__(self, obj):
+        self.warn()
+        self.decorator.__delete__(obj)
+
+    def __repr__(self):
+        return '<Deprecated attribute %s: %r>' % (
+            self.attr,
+            self.decorator)
+
+    def warn(self):
+        if not self.warning:
+            raise DeprecationWarning(
+                'The attribute %s is deprecated: %s' % (self.attr, self.message))
+        else:
+            warnings.warn(
+                'The attribute %s is deprecated: %s' % (self.attr, self.message),
+                DeprecationWarning,
+                stacklevel=3)
+
 def _parse_date(value):
     if not value:
         return None
@@ -389,17 +428,6 @@ def _serialize_list(value):
         return value
     return ', '.join(map(str, value))
 
-def _parse_user_agent(value):
-    return UserAgent(value or '')
-
-def _serialize_user_agent(value):
-    if value is None:
-        return None
-    value = str(value).strip()
-    if not value:
-        return None
-    return value
-
 def _parse_accept(value, header_name, AcceptClass, NilClass):
     if not value:
         return NilClass(header_name)
@@ -657,7 +685,7 @@ class Request(object):
         path = path.lstrip('/')
         return path.split('/', 1)[0]
 
-    def urlvars(self):
+    def urlvars__get(self):
         """
         Return any variables matched in the URL (e.g.,
         ``wsgiorg.routing_args``).
@@ -667,8 +695,21 @@ class Request(object):
         elif 'wsgiorg.routing_args' in self.environ:
             return self.environ['wsgiorg.routing_args'][1]
         else:
-            return {}
-    urlvars = property(urlvars, doc=urlvars.__doc__)
+            result = {}
+            self.environ['wsgiorg.routing_args'] = ((), result)
+            return result
+
+    def urlvars__set(self, value):
+        del self.urlvars
+        self.environ['wsgiorg.routing_args'] = ((), value)
+
+    def urlvars__del(self):
+        if 'paste.urlvars' in self.environ:
+            del self.environ['paste.urlvars']
+        if 'wsgiorg.routing_args' in self.environ:
+            del self.environ['wsgiorg.routing_args']
+            
+    urlvars = property(urlvars__get, urlvars__set, urlvars__del, doc=urlvars__get.__doc__)
 
     def is_xhr(self):
         """Returns a boolean if X-Requested-With is present and ``XMLHttpRequest``"""
@@ -725,7 +766,7 @@ class Request(object):
 
     body = property(_body__get, _body__set, _body__del, doc=_body__get.__doc__)
 
-    def str_postvars(self):
+    def str_POST(self):
         """
         Return a MultiDict containing all the variables from a POST
         form request.  Does *not* return anything for non-POST
@@ -768,26 +809,28 @@ class Request(object):
         env['webob._parsed_post_vars'] = (vars, self.body_file)
         return vars
 
-    str_postvars = property(str_postvars, doc=str_postvars.__doc__)
+    str_POST = property(str_POST, doc=str_POST.__doc__)
 
-    str_POST = str_postvars
+    str_postvars = deprecated_property(str_POST, 'str_postvars',
+                                       'use str_POST instead')
 
-    def postvars(self):
+    def POST(self):
         """
-        Like ``.str_postvars``, but may decode values and keys
+        Like ``.str_POST``, but may decode values and keys
         """
-        vars = self.str_postvars
+        vars = self.str_POST
         if self.charset:
             vars = UnicodeMultiDict(vars, encoding=self.charset,
                                     errors=self.unicode_errors,
                                     decode_keys=self.decode_param_names)
         return vars
 
-    postvars = property(postvars, doc=postvars.__doc__)
+    POST = property(POST, doc=POST.__doc__)
 
-    POST = postvars
+    postvars = deprecated_property(POST, 'postvars',
+                                   'use POST instead')
 
-    def str_queryvars(self):
+    def str_GET(self):
         """
         Return a MultiDict containing all the variables from the
         QUERY_STRING.
@@ -807,31 +850,34 @@ class Request(object):
         env['webob._parsed_query_vars'] = (vars, source)
         return vars
 
-    str_queryvars = property(str_queryvars, doc=str_queryvars.__doc__)
+    str_GET = property(str_GET, doc=str_GET.__doc__)
 
-    str_GET = str_queryvars
+    str_queryvars = deprecated_property(str_GET, 'str_queryvars',
+                                        'use str_GET instead')
+                                        
 
-    def queryvars(self):
+    def GET(self):
         """
-        Like ``.str_queryvars``, but may decode values and keys
+        Like ``.str_GET``, but may decode values and keys
         """
-        vars = self.str_queryvars
+        vars = self.str_GET
         if self.charset:
             vars = UnicodeMultiDict(vars, encoding=self.charset,
                                     errors=self.unicode_errors,
                                     decode_keys=self.decode_param_names)
         return vars
 
-    queryvars = property(queryvars, doc=queryvars.__doc__)
+    GET = property(GET, doc=GET.__doc__)
 
-    GET = queryvars
+    queryvars = deprecated_property(GET, 'queryvars',
+                                    'use GET instead')
 
     def str_params(self):
         """
         A dictionary-like object containing both the parameters from
         the query string and request body.
         """
-        return NestedMultiDict(self.queryvars, self.postvars)
+        return NestedMultiDict(self.str_GET, self.str_POST)
 
     str_params = property(str_params, doc=str_params.__doc__)
 
@@ -882,9 +928,6 @@ class Request(object):
 
     cookies = property(cookies, doc=cookies.__doc__)
 
-    ## FIXME: there should be a way to turn the request into a GET as part
-    ## of the copy; often a copied request is a safe subrequest of some sort,
-    ## and only GET is really safe (HEAD being uncommon)
     def copy(self):
         """
         Copy the request and environment object.
@@ -903,6 +946,20 @@ class Request(object):
         env['wsgi.input'] = fileobj
         return self.__class__(env)
 
+    def copy_get(self):
+        """
+        Copies the request and environment object, but turning this request
+        into a GET along the way.  If this was a POST request (or any other verb)
+        then it becomes GET, and the request body is thrown away.
+        """
+        env = self.environ.copy()
+        env['wsgi.input'] = StringIO('')
+        env['CONTENT_LENGTH'] = '0'
+        if 'CONTENT_TYPE' in env:
+            del env['CONTENT_TYPE']
+        env['REQUEST_METHOD'] = 'GET'
+        return self.__class__(env)
+
     def remove_conditional_headers(self, remove_encoding=True):
         """
         Remove headers that make the request conditional.
@@ -1019,17 +1076,7 @@ class Request(object):
     referer = environ_getter('HTTP_REFERER', rfc_section='14.36')
     referrer = referer
 
-    def referer_search_query(self):
-        """
-        Return the search query used to reach this page, if there was
-        one.  Returns a string.  If not found, returns None.
-        """
-        return parse_search_query(self.referer)
-    referrer_search_query = referer_search_query
-
-    user_agent = converter(
-        environ_getter('HTTP_USER_AGENT', rfc_section='14.43'),
-        _parse_user_agent, _serialize_user_agent, 'UserAgent object')
+    user_agent = environ_getter('HTTP_USER_AGENT', rfc_section='14.43')
 
     def __repr__(self):
         msg = '<%s at %x %s %s>' % (
@@ -1088,7 +1135,7 @@ class Request(object):
     # Will be filled in later:
     ResponseClass = None
 
-    def get_response(self, application):
+    def get_response(self, application, catch_exc_info=False):
         """
         Like ``.call_application(application)``, except returns a
         response object with ``.status``, ``.headers``, and ``.body``
@@ -1097,7 +1144,13 @@ class Request(object):
         This will use ``self.ResponseClass`` to figure out the class
         of the response object to return.
         """
-        status, headers, app_iter = self.call_application(application)
+        if catch_exc_info:
+            status, headers, app_iter, exc_info = self.call_application(
+                application, catch_exc_info=True)
+            del exc_info
+        else:
+            status, headers, app_iter = self.call_application(
+                application, catch_exc_info=False)
         return self.ResponseClass(status, headers, app_iter=app_iter, request=self)
 
     #@classmethod
@@ -1324,7 +1377,7 @@ class Response(object):
             header = self.headers.pop('content-type')
         except KeyError:
             raise AttributeError(
-                "You cannot set the charset when on content-type is defined")
+                "You cannot set the charset when no content-type is defined")
         match = _CHARSET_RE.search(header)
         if match:
             header = header[:match.start()] + header[match.end():]
@@ -1513,8 +1566,6 @@ class Response(object):
         else:
             return self._app_iter
 
-    ## FIXME: maybe the content-length clearing should only happen if
-    ## self._app_iter is not None?
     def _app_iter__set(self, value):
         if self._body is not None:
             # Undo the automatically-set content-length
@@ -1529,7 +1580,8 @@ class Response(object):
     app_iter = property(_app_iter__get, _app_iter__set, _app_iter__del, doc=_app_iter__get.__doc__)
 
     def set_cookie(self, key, value='', max_age=None,
-                   path='/', domain=None, secure=None, httponly=False):
+                   path='/', domain=None, secure=None, httponly=False,
+                   version=None, comment=None):
         """
         Set (add) a cookie for the response
         """
@@ -1541,6 +1593,8 @@ class Response(object):
             ('domain', domain),
             ('secure', secure),
             ('HttpOnly', httponly),
+            ('version', version),
+            ('comment', comment),
             ]:
             if var_value is not None and var_value is not False:
                 cookies[key][var_name.replace('_', '-')] = str(var_value)
@@ -1705,11 +1759,6 @@ class Response(object):
         for name, value in kw.items():
             setattr(cache_control, name, value)
 
-    ## FIXME: I'd like some native support for gzipping the response
-    ## This seems non-trivial to me, since it involves changing things like
-    ## the response content type.  It would probably be some method; either an
-    ## in-place method like response.gzip(), or a copied response object like
-    ## response.gzipped().
     content_encoding = header_getter('Content-Encoding', rfc_section='14.11')
 
     content_language = converter(
@@ -1765,7 +1814,7 @@ class Response(object):
 
     server = header_getter('Server', rfc_section='14.38')
 
-    ## FIXME: I realize response.var += 'something' won't work.  It should.
+    ## FIXME: I realize response.vary += 'something' won't work.  It should.
     ## Maybe for all listy headers.
     vary = converter(
         header_getter('Vary', rfc_section='14.44'),
@@ -1821,8 +1870,6 @@ class Response(object):
         """
         WSGI application interface
         """
-        ## FIXME: I should watch out here for bad responses, e.g.,
-        ## incomplete headers or body, etc
         if self.conditional_response:
             return self.conditional_response_app(environ, start_response)
         start_response(self.status, self.headerlist)
